--	Copyright (C) 2004 by neuron
--	aagaande AT gmail DOT com
--	
--	This program is free software; you can redistribute it and--or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation; either version 2 of the License, or
--	(at your option) any later version.
--	
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--	
--	You should have received a copy of the GNU General Public License
--	along with this program; if not, write to the
--	Free Software Foundation, Inc.,
--	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

local nQuestLog = nQuestLog
local moduleName = "Tooltips"
local Tooltips = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1","AceConsole-2.0")

local Quixote = AceLibrary("Quixote-1.0")
local Crayon = AceLibrary("Crayon-2.0")

local MobMap_Installed = false
local MobInfo2_Installed = false -- Not currently in use

local L = AceLibrary("AceLocale-2.2"):new("Tooltips")

function Tooltips:OnModuleLoad()
	self.menuName = L["Tooltips"]
	self.description = L["Description"]
	
	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		["tooltipAnchor"] = "DEFAULT",
		["itemToolTip"] = true,
		["mobToolTip"] = true,
		["disableInRaidInstance"] = true,
		["useMobMap"] = true,
		["MobMapDropRateFilter"] = 0.01,
		["addMobMapDropRates"] = true,
	})
	
	if (MobMapDotParentFrame) then
		MobMap_Installed = true
		if (self.db.profile.useMobMap) then
			self:EnableMobMap(true)
		end
		-- Used to cache mob names and their associated loot in memory, thus reducing lag
		mobNameCache = {}
		mobLootCache = {}
	end
end

function Tooltips:ToggleEnabled(v)
	self.db.profile.enabled = v
	self:ToggleItemToolTip()
	self:ToggleMobToolTip()
end

function Tooltips:ToggleItemToolTip()
	if (self.db.profile.itemToolTip and not self.itemHooked) then
		self.itemHooked = true
		self:HookScript(GameTooltip, "OnTooltipSetItem")
	elseif (not self.db.profile.itemToolTip and self.itemHooked) then
		self.itemHooked = nil
		self:Unhook(GameTooltip, "OnTooltipSetItem")
	end
end

function Tooltips:ToggleMobToolTip()
	if (self.db.profile.mobToolTip and not self.mobHooked) then
		self.mobHooked = true
		self:HookScript(GameTooltip, "OnTooltipSetUnit")
	elseif (not self.db.profile.mobToolTip and self.mobHooked) then
		self.mobHooked = nil
		self:Unhook(GameTooltip, "OnTooltipSetUnit")
	end
end

function Tooltips:EnableMobMap(v)
	if (v) then
		if type(MobMap_LoadDatabase) == "function" then
			if (not IsAddOnLoaded("MobMapDatabaseStub"..MOBMAP_DROP_DATABASE)) then
				MobMap_LoadDatabase(MOBMAP_DROP_DATABASE)
			end
			if (not IsAddOnLoaded("MobMapDatabaseStub"..MOBMAP_MOBNAME_DATABASE)) then
				MobMap_LoadDatabase(MOBMAP_MOBNAME_DATABASE)
			end
		end
		if (type(MobMap_GetIDForMobName) ~= "function") or (type(MobMap_GetMobLootTable) ~= "function") then
			MobMap_Installed = false
		end
	end
end

-- Most of the code for this function came from QuestIon
-- Not currently in use, due to MobMap
local function GetMI2DropChance(name, level, oDesc)
	local mobData = MI2_GetMobData(name, level)
	local combined = {}
	local dropChance = ""
	MI2_AddTwoMobs(combined, mobData);
	for levelToCombine = level - 4, level + 4, 1 do
		if levelToCombine ~= level then
			if MobInfoDB[name..":"..level] then
				local dataToCombine = MI2_GetMobData(name, levelToCombine)
				MI2_AddTwoMobs(combined, dataToCombine)
			end
		end
	end
	mobData = combined
	if (not mobData.itemList) then
		mobData.itemList = {};
	end
	for k, v in pairs(mobData.itemList) do
		local itemName = GetItemInfo(k)
		if (itemName) then
			if string.find(oDesc, itemName) then
				dropChance = v / mobData.loots * 100
				if(dropChance < 1) then
					dropChance = strsub(tostring(dropChance), 1, 4).."%"
				elseif(dropChance < 10) then
					dropChance = strsub(tostring(dropChance), 1, 3).."%"
				elseif(dropChance == 100) then
					dropChance = dropChance.."%"
				else
					dropChance = strsub(tostring(dropChance), 1, 2).."%"
				end
				dropChance = "["..dropChance.."] "
			end
		end
	end
	return dropChance
end

-- Mouseover tooltips

function Tooltips:OnTooltipSetUnit(tooltip, ...)
	local _,instanceType = IsInInstance()
	-- Disable tooltip parsing while in a raid instance, unless the option is unchecked in the menu
	if (instanceType ~= "raid") or (not self.db.profile.disableInRaidInstance) then
		-- Make sure that the unit isn't a player and is attackable before processing quest objectives
		if (not UnitPlayerControlled("mouseover")) and (UnitCanAttack("player","mouseover")) then
			local mobName = tooltip:GetUnit()
			if (mobName) then
				if Quixote:IsQuestMob(mobName) then
					for _,q,i,n in Quixote:IterateQuestsForMob(mobName) do
						local r,g,b = Crayon:GetThresholdColor(i/n)
						tooltip:AddDoubleLine(q,i..'/'..n,r,g,b,r,g,b)
					end
				end
				if (MobMap_Installed) and (self.db.profile.useMobMap) then
					local mobIsCached, mobLootCacheStart, mobLootCacheEnd = false, 0, 0
					-- Check to see if the mob has been cached in memory.  If found, set mobIsCached to true and
					-- set the starting and ending indexes for the mobLootCache table stored in memory for that mob
					if (table.getn(mobNameCache) > 0) then
						for mobIndex=1,table.getn(mobNameCache),1 do
							if (mobNameCache[mobIndex].mobName == mobName) then
								mobIsCached = true
								mobLootCacheStart = mobNameCache[mobIndex].lootIndexStart
								mobLootCacheEnd = mobNameCache[mobIndex].lootIndexEnd
							end
						end
					end
					local lootTable,lootTableRow = {},1
					-- If mob is cached in memory, get the cached loot table
					-- Otherwise, get it from MobMap and add it to the cache
					if (mobIsCached) then
						for lootIndex=mobLootCacheStart,mobLootCacheEnd,1 do
							-- Filter loot table and remove items whose drop rates are lower than the threshold chosen (default 1%)
							if ((mobLootCache[lootIndex].chance/100) >= self.db.profile.MobMapDropRateFilter) then
								lootTable[lootTableRow] = mobLootCache[lootIndex]
								lootTableRow = lootTableRow + 1
							end
						end
					else
						local mobID = MobMap_GetIDForMobName(mobName)
						if (mobID) then
							local mobLoot = MobMap_GetMobLootTable(mobID)
							if (mobLoot) then
								local lootIndexStart, lootIndexEnd
								if (table.getn(mobLootCache) > 0) then
									lootIndexStart = table.getn(mobLootCache) + 1
								else
									lootIndexStart = 1
								end
								lootIndexEnd = lootIndexStart + table.getn(mobLoot) - 1
								-- Filter loot table and remove items whose drop rates are lower than the threshold chosen (default 1%)
								for lootIndex=1,table.getn(mobLoot),1 do
									if ((mobLoot[lootIndex].chance/100) >= self.db.profile.MobMapDropRateFilter) then
										lootTable[lootTableRow] = mobLoot[lootIndex]
										lootTableRow = lootTableRow + 1
									end
									-- Add mob's loot table to the cache stored in memory
									table.insert(mobLootCache, mobLoot[lootIndex])
								end
								-- Add mob's name and loot table start/end rows to the cache stored in memory
								table.insert(mobNameCache,{mobName=mobName, lootIndexStart=lootIndexStart, lootIndexEnd=lootIndexEnd})
							end
						end
					end
					-- If the mob's loot table contains any items after filtering
					if (table.getn(lootTable) > 0) then
						-- Iterate through each quest, and then through each objective
						for qID,_,_,_,_,_,qObjs in Quixote:IterateQuests() do
							for oIndex=1,qObjs do
								local oDesc,oType,oGot,oNeeded,_ = Quixote:GetQuestStatusById(qID,oIndex)
								-- Only process objectives of type "item" or "object"
								if (oType == "item") or (oType == "object") then
									-- Iterate through mob's loot item table
									for lootIndex=1,table.getn(lootTable),1 do
										-- If loot item matches quest objective
										if (lootTable[lootIndex].itemname == oDesc) then
											local dropRate = ""
											-- Add MobMap item drop rate, if enabled
											if (self.db.profile.addMobMapDropRates) then
												dropRate = " ("..lootTable[lootIndex].chance.."%)"
											end
											-- Add objective, drop rate (if enabled), and completion status, colored by % complete, to tooltip
											local r,g,b = Crayon:GetThresholdColor(oGot/oNeeded)
											tooltip:AddDoubleLine(oDesc..dropRate,oGot.."/"..oNeeded,r,g,b,r,g,b)
										end
									end
								end
							end
						end
					end
				end
				tooltip:Show()
			end
		end
	end
	return self.hooks[tooltip].OnTooltipSetUnit(tooltip, ...)
end

function Tooltips:OnTooltipSetItem(tooltip, ...)
	local item = tooltip:GetItem()
	if (not item) then return end
	if Quixote:IsQuestItem(item) then
		for qID, qTitle, qLevel, _, qGroup, _, qObjs in Quixote:IterateQuests() do
			for oIndex = 1,qObjs do 
				local oDesc, _, oGot, oNeeded = Quixote:GetQuestStatusById(qID,oIndex)
				if oDesc == item then
					local qColor = GetDifficultyColor(qLevel)
					if qGroup > 0 then qLevel = qLevel.."G"..qGroup end
					tooltip:AddDoubleLine("["..qLevel.."] "..qTitle,oGot.."/"..oNeeded,qColor.r,qColor.g,qColor.b,Crayon:GetThresholdColor(oGot/oNeeded))
					tooltip:Show()
				end
			end
		end
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...)
end

-- Minion tooltips

function Tooltips:ShowUnlockedToolTip()
	if (self.db.profile["tooltipAnchor"] == "DEFAULT") then
		GameTooltip_SetDefaultAnchor(GameTooltip, nQuestLogFrame)
	else
		GameTooltip:SetOwner(nQuestLogFrame,self.db.profile["tooltipAnchor"])
	end
	
	GameTooltip:SetText(L["MinionUnlocked"])
	GameTooltip:AddLine(L["PlaceMinion"],1,1,1,1)
	GameTooltip:Show()
end

function Tooltips:ShowZoneToolTip(zone)
	if (self.db.profile["tooltipAnchor"] == "DEFAULT") then
		GameTooltip_SetDefaultAnchor(GameTooltip, nQuestLogFrame)
	else
		GameTooltip:SetOwner(nQuestLogFrame,self.db.profile["tooltipAnchor"])
	end
	
	--Set Title text
	GameTooltip:SetText(zone)
	
	local function showZoneQuests(showHidden)
		local quests = {}
		local nQuests = 0
		for _,iQuestId in Quixote:QuestsByZone(zone) do
			local title, iLevel, strTag, iGroup, iComplete, iObjectives = Quixote:GetQuestById(iQuestId)
			local strObjective = Quixote:GetQuestText(iQuestId)
			
			if (showHidden and not self.core.db.char.watchedQuests[title]) or (not showHidden and self.core.db.char.watchedQuests[title]) then
				local objectives,iTotalGot,iTotalNeeded = self.core:Format_QuestObjectives(title,iObjectives,strObjective,iQuestId,true)
				title = self.core:Format_QuestTitle(title,iLevel,strTag,iTotalGot,iTotalNeeded,iComplete)
				nQuests = nQuests + 1
				quests[title] = true
			end
		end
		return quests,nQuests
	end
	
	local function printQuests(quests)
		for name in pairs(quests) do
			GameTooltip:AddLine(name,1,0.5,0.5,0)
		end
	end
	
	if (self.core.db.char.hiddenZones[zone]) then
		GameTooltip:AddLine("  -  " .. L["ZoneHidden"],1,1,1,0)
	end
	
	local quests,nQuests = showZoneQuests(false)
	if (nQuests > 0) then
		GameTooltip:AddLine(L["TrackedQuests"] .. " (" .. nQuests .. ")",1,0.5,0.5,0)
		printQuests(quests)
	end
	
	local quests,nQuests = showZoneQuests(true)
	if (nQuests > 0) then
		GameTooltip:AddLine(L["HiddenQuests"] .. " (" .. nQuests .. ")",1,0.5,0.5,0)
		printQuests(quests)
	end
	
	GameTooltip:AddLine("\n" .. L["ZoneTips"],1,1,1,0)
	
	GameTooltip:Show()
end

function Tooltips:ShowQuestToolTip(iQuestId)
	-- TODO : layout options, ShowObjectives,ShowAll,ShowShort?
	-- TODO : maybe for example show all quest data on modifier key?
	local title, iLevel, strTag, iGroup, _, iObjectives, zone = Quixote:GetQuestById(iQuestId)
	local strObjective = Quixote:GetQuestText(iQuestId)
	
	if (title == nil) then
		return
	end
	
	if (self.db.profile["tooltipAnchor"] == "DEFAULT") then
		GameTooltip_SetDefaultAnchor(GameTooltip, nQuestLogFrame)
	else
		GameTooltip:SetOwner(nQuestLogFrame,self.db.profile["tooltipAnchor"])
	end
	
	--Set Title text
	GameTooltip:SetText(title)
	
	--Add objective description
	GameTooltip:AddLine(self.core:Colorize(strObjective,"objectiveText"),0.5,0.5,0.5,1)
	
	--Add progress
	local objectives,iTotalGot,iTotalNeeded = self.core:Format_QuestObjectives(title,iObjectives,strObjective,iQuestId,true)
	if (iTotalNeeded > 0) then
		local color = Crayon:GetThresholdHexColor(iTotalGot / iTotalNeeded)
		objectives = "\n" .. L["Total"] .. " : " .. Crayon:Colorize(color,iTotalGot .. "/" .. iTotalNeeded) .. objectives
	end
	GameTooltip:AddLine(objectives,1,1,1,0)
	
	--Add who in party has the quest:
	local ipartyMembers = GetNumPartyMembers()
	if (ipartyMembers > 0) then
		GameTooltip:AddLine("\n",1,1,1,0)
		for i = 1,ipartyMembers do
			if (IsUnitOnQuest(iQuestId, "party"..i)) then
				-- Party member is on the quest
				local strName = UnitName("party" .. i)
				
				-- Lets see if we can get quest progress too:
				local iPartyTotal = 0
				if (iObjectives > 0) then
					for i=1,iObjectives do
						local strDesc,strType, iGot, iNeeded = Quixote:GetQuestStatusById(iQuestId, i)
						local iPartyGot = Quixote:GetPartyQuestStatus(strName, iQuestId, strDesc)
						if (iPartyGot) then
							if (strType == "reputation") then 
								if (iNeeded == iPartyGot) then
									iPartyTotal = iPartyTotal + 1
								end
							else
								iPartyTotal = iPartyTotal + iPartyGot
							end
						end
					end
					local color = Crayon:GetThresholdHexColor(iPartyTotal / iTotalNeeded)
					strName = strName .. " " .. Crayon:Colorize(color,iPartyTotal .. "/" .. iTotalNeeded)
				end
				GameTooltip:AddLine(strName,0,1,0,0)
			else
				-- Party member is not on the quest
				GameTooltip:AddLine(UnitName("party" .. i),1,0,0,0)
			end
		end
	end
	
	GameTooltip:AddLine("\n" .. L["QuestTips"],1,1,1,0)
	GameTooltip:Show()
end

-- And the menu
function Tooltips:GetMenu(oldMenu)
	local menu = {
		type = "group",
		name = self.menuName,
		order = oldMenu.order,
		desc = self.description,
		args = {
			toggleEnabled = oldMenu,
			tooltipAnchor = {
				order = oldMenu.order + 1,
				type = "text",
				name = L["tooltipAnchor"],
				desc = L["tooltipAnchor desc"],
				validate={"DEFAULT","ANCHOR_TOPRIGHT", "ANCHOR_RIGHT", "ANCHOR_BOTTOMRIGHT","ANCHOR_TOPLEFT","ANCHOR_LEFT","ANCHOR_BOTTOMLEFT"},
				get = function()
					return self.db.profile.tooltipAnchor
				end,
				set = function(v)
					self.db.profile.tooltipAnchor = v
				end,
			},
			itemToolTip = {
				order = oldMenu.order + 2,
				type = "toggle",
				name = L["itemToolTip"],
				desc = L["itemToolTip desc"],
				get = function()
					return self.db.profile.itemToolTip
				end,
				set = function(v)
					self.db.profile.itemToolTip = v
					self:ToggleItemToolTip()
				end,
			},
			spacer1 = {
				type = "header",
				order = oldMenu.order + 3,
				name=" ",
			},
			mobToolTipTitle = {
				type = "header",
				order = oldMenu.order + 4,
				name = L["mobToolTip"],
			},
			mobToolTip = {
				order = oldMenu.order + 5,
				type = "toggle",
				name = L["Enabled"],
				desc = L["mobToolTip desc"],
				get = function()
					return self.db.profile.mobToolTip
				end,
				set = function(v)
					self.db.profile.mobToolTip = v
					self:ToggleMobToolTip()
				end,
			},
			disableInRaidInstance = {
				order = oldMenu.order + 6,
				type = "toggle",
				name = L["disableInRaidInstance"],
				desc = L["disableInRaidInstance desc"],
				get = function()
					return self.db.profile.disableInRaidInstance
				end,
				set = function(v)
					self.db.profile.disableInRaidInstance = v
				end,
			},
			useMobMap = {
				order = oldMenu.order + 7,
				type = "toggle",
				disabled = (not MobMap_Installed),
				name = L["useMobMap"],
				desc = L["useMobMap desc"],
				get = function()
					return self.db.profile.useMobMap
				end,
				set = function(v)
					self.db.profile.useMobMap = v
					self:EnableMobMap(v)
				end,
			},
			MobMapDropRateFilter = {
				order = oldMenu.order + 8,
				type = "range",
				hidden = (not MobMap_Installed),
				name = "- "..L["MobMapDropRateFilter"],
				desc = L["MobMapDropRateFilter desc"],
				get = function()
					return self.db.profile.MobMapDropRateFilter
				end,
				set = function(v)
					self.db.profile.MobMapDropRateFilter = v
				end,
				min = 0,
				max = 1,
				step = 0.01,
				isPercent = true,
			},
			addMobMapDropRates = {
				order = oldMenu.order + 9,
				type = "toggle",
				hidden = (not MobMap_Installed),
				name = "- "..L["addMobMapDropRates"],
				desc = L["addMobMapDropRates desc"],
				get = function()
					return self.db.profile.addMobMapDropRates
				end,
				set = function(v)
					self.db.profile.addMobMapDropRates = v
				end,
			},
		}
	}
	return menu
end
